@@ -94,8 +94,10 @@ html_engine_set_mark (HTMLEngine *e)
 	g_return_if_fail (HTML_IS_ENGINE (e));
 	g_return_if_fail (e->editable || e->caret_mode);
 
-	if (e->mark != NULL)
+	if (e->mark != NULL) {
 		html_engine_unselect_all (e);
+		html_cursor_destroy(e->mark);
+	}
 
 	e->mark = html_cursor_dup (e->cursor);
 
@@ -122,56 +124,64 @@ html_engine_clipboard_pop (HTMLEngine *e)
 	e->clipboard_stack = g_list_remove (e->clipboard_stack, e->clipboard_stack->data);
 }
 
+typedef struct 
+{
+	gboolean active;
+	gint cursor;
+	gint mark;
+} Selection;
+
 void
 html_engine_selection_push (HTMLEngine *e)
 {
+	Selection *selection = g_new (Selection, 1);
+
 	if (html_engine_is_selection_active (e)) {
-		e->selection_stack
-			= g_list_prepend (e->selection_stack, GINT_TO_POINTER (html_cursor_get_position (e->mark)));
-		e->selection_stack
-			= g_list_prepend (e->selection_stack, GINT_TO_POINTER (html_cursor_get_position (e->cursor)));
-		e->selection_stack = g_list_prepend (e->selection_stack, GINT_TO_POINTER (TRUE));
+		selection->active = TRUE;
+		selection->cursor = html_cursor_get_position (e->cursor);
+		selection->mark = html_cursor_get_position (e->mark);
 	} else {
-		e->selection_stack = g_list_prepend (e->selection_stack, GINT_TO_POINTER (FALSE));
+		selection->active = FALSE;
+		selection->cursor = -1;
+		selection->mark = -1;
 	}
+
+	e->selection_stack = g_list_prepend (e->selection_stack, selection);
 }
 
 void
 html_engine_selection_pop (HTMLEngine *e)
 {
-	gboolean selection;
+	Selection *selection;
 
 	g_assert (e->selection_stack);
 
-	selection = GPOINTER_TO_INT (e->selection_stack->data);
-	e->selection_stack = g_list_remove (e->selection_stack, e->selection_stack->data);
+	selection = e->selection_stack->data;
+	e->selection_stack = g_list_delete_link (e->selection_stack, e->selection_stack);
 
 	html_engine_disable_selection (e);
 
-	if (selection) {
-		gint cursor, mark;
-
-		cursor = GPOINTER_TO_INT (e->selection_stack->data);
-		e->selection_stack = g_list_remove (e->selection_stack, e->selection_stack->data);
-		mark = GPOINTER_TO_INT (e->selection_stack->data);
-		e->selection_stack = g_list_remove (e->selection_stack, e->selection_stack->data);
-
-		html_cursor_jump_to_position (e->cursor, e, mark);
+	if (selection->active) {
+		html_cursor_jump_to_position (e->cursor, e, selection->mark);
 		html_engine_set_mark (e);
-		html_cursor_jump_to_position (e->cursor, e, cursor);
+		html_cursor_jump_to_position (e->cursor, e, selection->cursor);
 	}
 	html_engine_edit_selection_updater_update_now (e->selection_updater);
+
+	g_free (selection);
 }
 
 gboolean
 html_engine_selection_stack_top (HTMLEngine *e, gint *cpos, gint *mpos)
 {
-	if (e->selection_stack && GPOINTER_TO_INT (e->selection_stack->data) && e->selection_stack->next && e->selection_stack->next->next) {
+	Selection *selection = e->selection_stack ? e->selection_stack->data : NULL;
+	
+	if (selection && selection->active) {
 		if (cpos)
-			*cpos = GPOINTER_TO_INT (e->selection_stack->next->data);
+			*cpos = selection->cursor;
 		if (mpos)
-			*mpos = GPOINTER_TO_INT (e->selection_stack->next->next->data);
-
+			*mpos = selection->mark;
+		
 		return TRUE;
 	}
 
@@ -181,15 +191,16 @@ html_engine_selection_stack_top (HTMLEngine *e, gint *cpos, gint *mpos)
 gboolean
 html_engine_selection_stack_top_modify (HTMLEngine *e, gint delta)
 {
-	if (e->selection_stack && GPOINTER_TO_INT (e->selection_stack->data) && e->selection_stack->next && e->selection_stack->next->next) {
-		e->selection_stack->next->data = GINT_TO_POINTER (GPOINTER_TO_INT (e->selection_stack->next->data) + delta);
-		e->selection_stack->next->next->data = GINT_TO_POINTER (GPOINTER_TO_INT (e->selection_stack->next->next->data) + delta);
-
+	Selection *selection = e->selection_stack ? e->selection_stack->data : NULL;
+	
+	if (selection && selection->active) {
+		selection->cursor += delta;
+		selection->mark += delta;
+			
 		return TRUE;
 	}
 
 	return FALSE;
-
 }
 
 static void
@@ -615,7 +626,6 @@ html_engine_fill_pre_line (HTMLEngine *e)
 	guint position;
 	guint line_offset;
 	guint last_space;
-	HTMLObject *flow;
 	gunichar uc;
 
 	g_assert (e->cursor->object);
@@ -631,7 +641,6 @@ html_engine_fill_pre_line (HTMLEngine *e)
 
 	html_cursor_beginning_of_paragraph (e->cursor, e);
 
-	flow = e->cursor->object->parent;
 	while (html_cursor_forward (e->cursor, e) && (e->cursor->position < position - 1)) {
 		uc = html_cursor_get_current_char (e->cursor);
 		line_offset = inc_line_offset (line_offset, uc);
@@ -773,8 +782,10 @@ html_engine_get_insert_level_for_object (HTMLEngine *e, HTMLObject *o)
 		if (e && e->cursor->object && e->cursor->object->parent && e->cursor->object->parent->parent && html_object_is_clue (e->cursor->object->parent->parent)) {
 			HTMLObject *clue = e->cursor->object->parent->parent;
 
-			while (clue && clue->parent && (HTML_IS_CLUEV (clue->parent) || HTML_IS_TABLE_CELL (clue->parent)))
-			       cursor_level ++;
+			while (clue && clue->parent && (HTML_IS_CLUEV (clue->parent) || HTML_IS_TABLE_CELL (clue->parent))) {
+				clue = clue->parent;
+				cursor_level ++;
+			}
 		}
 	}
 
