@@ -58,7 +58,7 @@ html_point_cursor_object_eq (HTMLPoint *p, HTMLPoint *c)
 inline void
 html_point_next_cursor (HTMLPoint *p)
 {
-	p->object = html_object_next_cursor (p->object, &p->offset);
+	p->object = html_object_next_cursor (p->object, (gint *) &p->offset);
 }
 
 HTMLInterval *
@@ -329,12 +329,6 @@ html_object_children_max (HTMLObject *a, HTMLObject *b)
 	return a;
 }
 
-static inline HTMLObject *
-html_object_children_min (HTMLObject *a, HTMLObject *b)
-{
-	return a == html_object_children_max (b, a) ? b : a;
-}
-
 HTMLPoint *
 html_point_max (HTMLPoint *a, HTMLPoint *b)
 {
